@@ -2,66 +2,54 @@
 
 import numpy
 import scipy
-from numpy import exp, sqrt, pi, sin
-from scipy.interpolate import PiecewisePolynomial, interp1d
-from scipy.special import gamma, gammainc, dawsn, hyp1f1
-from scipy.integrate import ode, quad, simps
-from math import factorial
+from scipy.interpolate import BPoly, interp1d, UnivariateSpline
+from numpy import exp, sqrt, pi, sin, cos, log10
+from scipy.special import gamma, gammainc, hyp1f1
+from scipy.integrate import ode, simps, quad
+from math import factorial, sinh
 
 #     Authors: Mark Gieles, Alice Zocchi (Surrey 2015)
 
 class limepy:
     def __init__(self, phi0, g, **kwargs):
         r"""
-
         (MM, A) LIMEPY
-
         (Multi-Mass, Anisotropic) Lowered Isothermal Model Explorer in Python
-
         This code solves the models presented in Gieles & Zocchi 2015 (GZ15),
         and calculates radial profiles for some useful quantities. The models
         are defined by the distribution function (DF) of equation (1) in GZ15.
-
         Model parameters:
         =================
-
         phi0 : scalar, required
              Central dimensionless potential
         g : scalar, required
           Order of truncation (0<= g < 3.5; 0=Woolley, 1=King, 2=Wilson)
-
         ra : scalar, required for anisotropic models
-          Anisotropy radius; default=1e8
+           Anisotropy radius; default=1e8
         mj : list, required for multi-mass system
-          Mean mass of each component; default=None
+           Mean mass of each component; default=None
         Mj : list, required for multi-mass system
            Total mass of each component; default=None
         delta : scalar, optional
               Index in s_j = s x mu_j^-delta; default=0.5
               See equation (24) in GZ15
         eta : scalar, optional
-              Index in ra_j = ra x mu_j^eta; default=0
-              See equation (25) in GZ15
-
+            Index in ra_j = ra x mu_j^eta; default=0
+            See equation (25) in GZ15
         Input for scaling:
         ==================
-
-        scale : bool, optional
-           Scale model to desired G=GS, M=MS, R=RS; default=False
-        MS : scalar, optional
-           Final scaled mass; default=10^5 [Msun]
-        RS : scalar, optional
-           Final scaled radius, half-mass or virial (see below); default=3 [pc]
-        GS : scalar, optional
-           Final scaled gravitationsl const; default=0.004302 [(km/s)^2 pc/Msun]
-        scale_radius : str, optional
-                     Radius to scale ['rv' or 'rh']; default='rh'
-
+        G : scalar, optional
+          Final scaled gravitationsl const; default=0.004302 [(km/s)^2 pc/Msun]
+        M : scalar, optional
+          Final scaled mass; default=10^5
+        r0, rh, rv, rt : scalar, optional
+                       Final scaled radius; default=rh=3
         Options:
         ========
-
         project : bool, optional
                 Compute model properties in projection; default=False
+        meanmassdef : string [global|central]
+                    Definition of <m> in mu_j = m_j/<m>; default='global'
         potonly : bool, optional
                 Fast solution by solving potential only; default=False
         max_step : scalar, optional
@@ -70,14 +58,12 @@ class limepy:
                 Print diagnostics; default=False
         ode_atol : absolute tolerance parameter for ode solver; default=1e-7
         ode_rtol : relative tolerance parameter for ode solver; default=1e-7
-
         Output variables:
         =================
-
         All models:
         -----------
          rhat, phihat, rhohat : radius, potential and density in model units
-         r, phi, rho : as above, in scaled units (if scale=True)
+         r, phi, rho : as above, in scaled units
          v2, v2r, v2t : total, radial and tangential mean-square velocity
          beta : anisotropy profile (equation 32, GZ15)
          mc : enclosed mass profile
@@ -88,14 +74,12 @@ class limepy:
          volume : phase-space volume occupied by model
          nstep : number of integration steps (depends on ode_rtol & ode_atol)
          converged : bool flag to indicate whether model was solved
-
         Projected models:
         -----------------
          Sigma : surface (mass) density
-         v2z : line-of-sight mean-square velocity
+         v2p : line-of-sight mean-square velocity
          v2R, v2T : radial and tangential component of mean-square velocity
                   on plane of the sky
-
         Multi-mass models:
         ------------------
          Properties of each component:
@@ -108,79 +92,99 @@ class limepy:
          r0j, raj : radii (King, anisotropy)
          Kj : kinetic energy
          Krj, Ktj : radial/tangential component of kinetic energy
-
         Projected multi-mass models:
         ---------------------------
          Properties of each component:
          Sigmaj : surface (mass) density
-         v2zj : line-of-sight mean-square velocity profile
+         v2pj : line-of-sight mean-square velocity profile
          v2Rj, v2Tj : radial and tangential component on the plane of the sky
                     of the mean-square velocity profile
-
         Examples:
         =========
-
         Construct a Woolley model with phi0 = 7 and print r_t/r_0 and r_v/r_h
-
         >>> k = limepy(7, 0)
         >>> print k.rt/k.r0, k.rv/k.rh
         >>> 19.1293426415 1.17783663028
-
         Construct a Michie-King model and print ratio of anisotropy radius over
         half-mass radius and the Polyachenko & Shukhman (1981) anisotropy
         parameter
-
         >>> a = limepy(7, 1, ra=5)
         >>> print a.ra/a.rh, 2*a.Kr/a.Kt
         >>> 1.03377960149 1.36280949941
-
         Create a Wilson model with phi0 = 12 in Henon/N-body units: G = M =
         r_v = 1 and print the normalisation constant A of the DF and the
         value of the DF in the centre:
-
-        >>> w = limepy(12, 2, scale=True, GS=1, MS=1, RS=1, scale_radius='rv')
+        >>> w = limepy(12, 2, G=1, M=1, rv=1)
         >>> print w.A, w.df(0,0)
         >>> [ 0.00800902] [ 1303.40270676]
-
         Multi-mass model in physical units with r_h = 3 pc and M = 10^5 M_sun
         and print central densities of each bin over the total central density
         and the half-mass radius + half-mass radius in projection
-
-        >>> m = limepy(7, 1, mj=[0.3,1,5], Mj=[9,3,1], scale=True, project=True)
+        >>> m = limepy(7, 1, mj=[0.3,1,5], Mj=[9,3,1], rh=3, M=1e5,project=True,\
+        ...: meanmassdef='central')
         >>> print m.alpha, m.rh, m.rhp
         >>> [ 0.30721416  0.14103549  0.55175035] 3.0 2.25494426759
-
         """
 
         # Set parameters and scales
         self._set_kwargs(phi0, g, **kwargs)
         self.rhoint0 = [self._rhoint(self.phi0, 0, self.ramax)]
 
+
         # In case of multi-mass model, iterate to find central densities
         # (see Section 2.2 in GZ15)
+
         if (self.multi):
+            # Feb 2016: New method to iterate on the MF.
+            # First solve isotropic model
+
+            ratmp = self.ra*1.0
+
+            self.ra = self.ramax*1.0
             self._init_multi(self.mj, self.Mj)
 
             while self.diff > self.diffcrit:
                 self._poisson(True)
-
                 if (not self.converged):
-                    error = "Error: model did not converge in first iteration,"
-                    error += " try larger r_a / smaller phi_0"
+                    error = "Error: rmax reached in mf iteration"
                     raise ValueError(error)
                 else:
                     self._set_alpha()
                     if self.niter > self.max_mf_iter:
                         self.converged=False
-                        error = "Error: mass function did not converge, "
-                        errpr += " try larger phi_0"
+                        error = "Error: maximum number of iterations reached "
                         raise ValueError(error)
 
+            # Now solve anisotropic multimass
+            if ratmp < self.ramax:
+                self.diff = 1
+                self.ra = ratmp*1.0
+                self._set_mass_function_variables()
+
+                while self.diff > self.diffcrit:
+                    self._poisson(True)
+                    if (not self.converged):
+                        error = "Error: rmax reached in mf iteration"
+                        raise ValueError(error)
+                    else:
+                        self._set_alpha()
+                        if self.niter > self.max_mf_iter:
+                            self.converged=False
+                            error ="Error: maximum number of iterations reached"
+                            raise ValueError(error)
+
+
         self.r0 = 1.0
         if (self.multi): self.r0j = sqrt(self.s2j)*self.r0
 
-        # Solve Poisson equation to get the potential
-        self._poisson(self.potonly)
+        # ROT
+        if (self.rot):
+            self._poisson_rot()
+
+        else:
+            # Solve Poisson equation to get the potential
+            self._poisson(self.potonly)
+
         if (self.multi): self.Mj = self._Mjtot
 
         # Optional scaling
@@ -215,17 +219,22 @@ class limepy:
         if (g<0): raise ValueError("Error: g must be larger or equal to 0")
         if (g>=3.5): raise ValueError("Error: for g>=3.5 models are infinite")
 
-        self.phi0, self.g = phi0, g
+        self.model = "limepy"
 
-        self.MS, self.RS, self.GS = 1e5, 3, 0.004302
-        self.scale_radius = 'rh'
+        # ROT
+        self.omega = 0
+
+        self.phi0, self.g = phi0, g
+        self._MS, self._RS, self._GS = None, None, None
+        self.scale_radius = None
         self.scale = False
         self.project = False
+        self.meanmassdef='global'
         self.maxr = 1e10
         self.max_step = self.maxr
         self.diffcrit = 1e-8
         self.max_arg_exp = 700  # Maximum argument for exponent and hyp1f1 func
-        self.max_mf_iter = 200  # Maximum number of iterations to find rho0j
+        self.max_mf_iter = 100  # Maximum number of iterations to find rho0j
         self.minimum_phi = 1e-8 # Stop criterion for integrator
         self.mf_iter_index = 0.5
         self.ode_atol = 1e-7
@@ -247,7 +256,56 @@ class limepy:
 
         if kwargs is not None:
             for key, value in kwargs.items():
-                setattr(self, key, value)
+                # Check for scaling input
+                if key is 'G':
+                    self._GS, self.scale = value, True
+                elif key is 'M':
+                    self._MS, self.scale = value, True
+                elif key is 'r0':
+                    if self.scale_radius is None:
+                        self._RS,self.scale_radius,self.scale = value,'r0', True
+                    else:
+                        error="Can not set scale radius to r0,already set to %s"
+                        raise ValueError(error%self.scale_radius)
+                elif key is 'rh':
+                    if self.scale_radius is None:
+                        self._RS, self.scale_radius, self.scale=value,'rh', True
+                    else:
+                        error="Can not set scale radius to rh,already set to %s"
+                        raise ValueError(error%self.scale_radius)
+                elif key is 'rv':
+                    if self.scale_radius is None:
+                        self._RS, self.scale_radius, self.scale=value,'rv', True
+                    else:
+                        error="Can not set scale radius to rv,already set to %s"
+                        raise ValueError(error%self.scale_radius)
+                elif key is 'rt':
+                    if self.scale_radius is None:
+                        self._RS, self.scale_radius, self.scale=value,'rt', True
+                    else:
+                        error="Can not set scale radius to rt,already set to %s"
+                        raise ValueError(error%self.scale_radius)
+                else:
+                    # Set input parameters
+                    setattr(self, key, value)
+
+            if (self.scale):
+                if self._MS is None:
+                    self._MS = 1e5
+                    if (self.verbose):
+                        print(" No mass-scale provided, set to default M = 1e5")
+                if self._RS is None:
+                    self._RS, self.scale_radius = 3, 'rh'
+                    if (self.verbose):
+                        print(" No radius-scale provided, set to default rh = 3")
+                if self._GS is None:
+                    self._GS = 0.004302
+                    if (self.verbose):
+                        print(" No G provided, set to default: G = 0.004302")
+                if (self.verbose):
+                    vars=(self._GS, self._MS, self.scale_radius, self._RS)
+                    print(" Model scaled to: G = %s, M = %s, %s = %s"%vars)
+
             if 'mj' in kwargs and 'Mj' in kwargs:
                 self.multi=True
                 if len(self.Mj) is not len(self.mj):
@@ -257,6 +315,13 @@ class limepy:
                 raise ValueError("Error: Supply both mj and Mj")
         self.raj = numpy.array([self.ra])
 
+        if self.potonly and self.project:
+            raise ValueError('Error: You must not use potonly and project')
+
+        self.rot = False
+        if self.omega > 0:
+            print(" Warning: ROTATION PART NOT FINISHED! ")
+            self.rot = True
         return
 
     def _logcheck(self, t, y):
@@ -269,10 +334,17 @@ class limepy:
     def _set_mass_function_variables(self):
         """ Multi-mass models: Set properties for each mass bin """
 
-        self.mmean = sum(self.mj*self.alpha)    # equation (26) GZ15
+        if self.meanmassdef=='global':
+            Nj = self.Mj/self.mj
+            self.mmean = sum(self.Mj/sum(Nj))
+        elif self.meanmassdef=='central':
+            self.mmean = sum(self.mj*self.alpha)    # equation (26) GZ15
+        else:
+            raise ValueError(" meanmass must be 'global' or 'central'")
+
         self.mu = self.mj/self.mmean
-        self.s2j = self.mu**(-2*self.delta)     # equation (24) GZ15
-        self.raj = self.ra*self.mu**self.eta    # equation (25) GZ15
+        self.s2j = self.mu**(-2*self.delta)         # equation (24) GZ15
+        self.raj = self.ra*self.mu**self.eta        # equation (25) GZ15
 
         self.phi0j = self.phi0/self.s2j
         self.rhoint0 = numpy.zeros(self.nmbin)
@@ -300,11 +372,10 @@ class limepy:
     def _set_alpha(self):
         """ Set central rho_j for next iteration """
 
-        # The power of mf_iter_index < 1 is used. This is different from the
-        # recommendation of Da Costa & Freeman 1976 and Gunn & Griffin 1979:
-        # mf_iter_index = 1, a smaller value leads to better convergens for
-        # low phi0 and wide MFs (i.e. when black-holes are considered), which
-        # can fail with an index of 1 (see Section 2.2, GZ15)
+        # The power of mf_iter_index = 0.5. This is lower than the recommended
+        # value of 1 as used in Da Costa & Freeman 1976 and Gunn & Griffin 1979.
+        # Better convergence is reached for a smaller value, but a user may
+        # experiment with larger values to gain speed.
 
         self.alpha *= (self.Mj/self._Mjtot)**self.mf_iter_index
         self.alpha/=sum(self.alpha)
@@ -321,9 +392,10 @@ class limepy:
                 M2 = self.Mj[j]/sum(self.Mj)
                 fracd=fracd+"%7.3f "%(( M1 - M2)/M2)
                 Mjit=Mjit+"%7.3f "%(self._Mjtot[j]/sum(self._Mjtot))
-            out = (self.niter, self.diff, self.converged, fracd, Mjit)
-            frm = " Iter %3i; diff = %8.1e; conv = %s;"
-            frm += " frac diff=%s; Mjtot=%s"
+            out = (self.niter, self.mf_iter_index, self.diff, self.converged*1,\
+                   fracd, Mjit)
+            frm = " %2i; ind=%3.1f; diff= %6.1e; conv= %s;"
+            frm += " fdiff=%s; Mjtot=%s"
             print(frm%out)
 
     def _poisson(self, potonly):
@@ -409,9 +481,8 @@ class limepy:
             rhotmp += self.alpha[j]*self._rhohat(phi, self.r[ih:ih+2], j)
         drdm = 1./(4*pi*self.r[ih:ih+2]**2*rhotmp)
         rmc_and_derivs = numpy.vstack([[self.r[ih:ih+2]],[drdm]]).T
-        self.rh = float(PiecewisePolynomial(self.mc[ih:ih+2], rmc_and_derivs,
-                                      direction=1)(0.5*self.mc[-1]))
 
+        self.rh = BPoly.from_derivatives(self.mc[ih:ih+2], rmc_and_derivs)(0.5*self.mc[-1])
         self.rv = -0.5*self.G*self.M**2/self.U
 
         # Additional stuff
@@ -485,7 +556,6 @@ class limepy:
             # Calculate anisotropy profile (equation 32 of GZ15)
             self.beta = self._beta(self.r, self.v2r, self.v2t)
 
-
     def _rhohat(self, phi, r, j):
         """
         Wrapper for _rhoint when either: both phi or r are arrays, or both
@@ -615,9 +685,87 @@ class limepy:
 
         dVdvdr = (4*pi)**2*x**2 * (2*y[0])**1.5/3 if (x>0) and (y[0]>0) else 0
         derivs.append(dVdvdr)
-
         return derivs
 
+
+    # ROT: UNDER CONSTRUCTIONS  ##############
+    def _poisson_rot(self):
+        self._init_rot()
+        diff_rot = 1e99
+        while (diff_rot > self.diff_rot_crit):
+            # Compute density
+            rho_l = self._rhohat_rot()
+            self.rho_l = rho_l
+            # Compute potential
+
+            diff_rot = 0
+
+    def _phil_rot(self):
+        phi_l = numpy.zeros(len(self.r)*nl).reshape(len(self.r), nl)
+
+        phi_l[:,0] = self.phi0*sqrt(2)
+
+    def legendre_Ul(self, l, theta):
+        il = l/2.0
+        dl = 1.0/2**l
+        m = numpy.arange(il+1)
+        cm = gamma(2*l-2*m+1)*(-1)**m/(gamma(m+1)*gamma(l-m+1)*gamma(l-2*m+1))
+        gm = cos(theta)**(l-2*m)
+        Pl = dl*numpy.sum(cm*gm)
+        return sqrt((2*l+1)/2)*Pl
+
+    def _rhohat_rot(self):
+        for ip in range(len(self.r)):
+            for it in range(self.ntheta):
+                self.rhon_g[ip][it] = self._rhoint_rot(self.phin_g[ip][it], self.r[ip], self.theta[it])
+
+        nl = int(self.lmax/2+1)
+        rho_l = numpy.zeros(len(self.r)*nl).reshape(len(self.r), nl)
+
+        for il in range(nl):
+            l = 2*il
+            for ir in range(len(self.r)):
+                Ul = numpy.zeros(len(self.theta))
+                for it in range(self.ntheta):
+                    Ul[it] = self.legendre_Ul(l, self.theta[it])
+                rho_l[ir][il] = -2.0 * simps(self.rhon_g[ir]*Ul, x=cos(self.theta))
+
+        return rho_l
+
+    def _rhoint_rot(self, phi, r, theta):
+        # Compute eq 3 for scalar phi, r and theta
+        g, om = self.g, self.omega
+        Q = 3*sqrt(2)*r*sin(theta)
+        # Eq 3
+        integ = quad(lambda x: exp(phi-x)*gammainc(g, phi-x)*numpy.sinh(om*Q*x**0.5), 0, phi)[0]
+
+        return integ/(om*Q*gamma(1.5))
+
+    def _init_rot(self):
+        self.lmax = 6
+        self.diff_rot_crit = 1e-3
+
+        # Grid params
+        self.ntheta = (2*self.lmax + 1)
+
+        # Compute spherical model first
+        self._poisson(True)
+        phin = self.phi
+        self.theta = numpy.linspace(1e-4, 0.5*pi, self.ntheta)
+
+        # Set up grid
+        self.r_g, self.theta_g = numpy.meshgrid(self.r, self.theta, indexing ='ij')
+        self.phin_g, tmp = numpy.meshgrid(self.phi, self.theta, indexing ='ij')
+        self.rhon_g = self.phin_g*0.0
+
+        # Fudge TBD
+        self.r[0] = 0.5*self.r[1]
+
+        # Make copy for iterations
+        self.phin_prev_g = self.phin_g*1.0
+        return
+    ################
+
     def _setup_phi_interpolator(self):
         """ Setup interpolater for phi, works on scalar and arrays """
 
@@ -629,19 +777,21 @@ class limepy:
             phi_and_derivs = numpy.vstack([[self.phi],[self.dphidr1]]).T
         else:
             phi_and_derivs = numpy.vstack([[self.phihat],[self.dphidrhat1]]).T
-        self._phi_poly = PiecewisePolynomial(self.r,phi_and_derivs,direction=1)
+
+        self._phi_poly = BPoly.from_derivatives(self.r,phi_and_derivs)
 
     def _scale(self):
         """ Scales the model to the units set in the input: GS, MS, RS """
 
-        Mstar = self.MS/self.M
-        Gstar = self.GS/self.G
-        if (self.scale_radius=='rh'): Rstar = self.RS/self.rh
-        if (self.scale_radius=='rv'): Rstar = self.RS/self.rv
+        Mstar = self._MS/self.M
+        Gstar = self._GS/self.G
+        if (self.scale_radius=='r0'): Rstar = self._RS/self.r0
+        if (self.scale_radius=='rh'): Rstar = self._RS/self.rh
+        if (self.scale_radius=='rv'): Rstar = self._RS/self.rv
+        if (self.scale_radius=='rt'): Rstar = self._RS/self.rt
         v2star =  Gstar*Mstar/Rstar
 
         # Update the scales that define the system (see Section 2.1.2 of GZ15)
-
         self.G *= Gstar
         self.rs = Rstar
         self.s2 *= v2star
@@ -665,11 +815,10 @@ class limepy:
         self.A *= Mstar/(v2star**1.5*Rstar**3)
         self.volume *= v2star**1.5*Rstar**3
 
-        if (self.multi):
-            self.Mj *= Mstar
-            self.r0j *= Rstar
 
         # Scale density, velocity dispersion components, kinetic energy
+        self.rho = self.rhohat*Mstar/Rstar**3
+
         if (not self.potonly):
             self.rho = self.rhohat*Mstar/Rstar**3
             self.v2, self.v2r, self.v2t = (q*v2star for q in [self.v2,
@@ -681,6 +830,8 @@ class limepy:
                 self.rhoj = self.rhohatj * Mstar/Rstar**3
                 for j in range(self.nmbin):
                     self.rhoj[j] *= self.alpha[j]
+                self.Mj *= Mstar # 28/1/19 Thanks to William
+
                 self.mcj *= Mstar
                 self.rhj *= Rstar
                 self.v2j,self.v2rj,self.v2tj=(q*v2star for q in
@@ -702,21 +853,21 @@ class limepy:
 
         # Initialise the projected quantities:
         # R is the projected (2d) distance from the center, Sigma is the
-        # projected density, v2z is the line-of-sight velocity dispersion,
+        # projected density, v2p is the line-of-sight velocity dispersion,
         # v2R and v2T are the radial and tangential velocity dispersion
         # components projected on the plane of the sky
 
         # Initialise some arrays
         R = self.r
         Sigma = numpy.zeros(self.nstep)
-        v2z = numpy.zeros(self.nstep)
+        v2p = numpy.zeros(self.nstep)
         v2R = numpy.zeros(self.nstep)
         v2T = numpy.zeros(self.nstep)
         mcp = numpy.zeros(self.nstep)
 
         if (self.multi):
             Sigmaj = numpy.zeros((self.nmbin, self.nstep))
-            v2zj = numpy.zeros((self.nmbin, self.nstep))
+            v2pj = numpy.zeros((self.nmbin, self.nstep))
             v2Rj = numpy.zeros((self.nmbin, self.nstep))
             v2Tj = numpy.zeros((self.nmbin, self.nstep))
             mcpj = numpy.zeros((self.nmbin, self.nstep)) # TBD
@@ -727,16 +878,21 @@ class limepy:
             r = self.r[c]
             z = sqrt(abs(r**2 - R[i]**2)) # avoid small neg. values
 
-            Sigma[i] = 2.0*simps(self.rho[c], x=z)
+            Sigma[i] = 2.0*abs(simps(self.rho[c], x=z))
+
             betaterm1 = 1 if i==0 else 1-self.beta[c]*R[i]**2/self.r[c]**2
-            betaterm2 = 1 if i==0 else 1-self.beta[c]*(1-R[i]**2/self.r[c]**2)
-            v2z[i] = abs(2.0*simps(betaterm1*self.rho[c]*self.v2r[c], x=z))
-            v2z[i] /= Sigma[i]
+
+            # eq 41 in paper has a small mistake:  (1-R^2)/r^2 should be (1-R^2/r^2) as below
+            betaterm2 = 1 - self.beta[c] if i==0 else 1-self.beta[c]*(1-R[i]**2/self.r[c]**2)
+
+            v2p[i] = abs(2.0*simps(betaterm1*self.rho[c]*self.v2r[c], x=z))
+            v2p[i] /= Sigma[i]
 
             v2R[i] = abs(2.0*simps(betaterm2*self.rho[c]*self.v2r[c], x=z))
             v2R[i] /= Sigma[i]
 
-            v2T[i] = abs(2.0*simps(self.rho[c]*self.v2t[c]/2., x=z)/Sigma[i])
+            v2T[i] = abs(simps(self.rho[c]*self.v2t[c], x=z))
+            v2T[i] /= Sigma[i]
 
             # Cumulative mass in projection
             if (i>0):
@@ -752,14 +908,15 @@ class limepy:
                     Sigmaj[j,i] = 2.0*simps(self.rhoj[j,c], x=z)
                     if (i==0):
                         betaterm1 = 1
-                        betaterm2 = 1
+                        betaterm2 = 1 -self.betaj[j,c]
                     else:
                         betaterm1 = 1-self.betaj[j,c]*R[i]**2/self.r[c]**2
-                        betaterm2 = 1-self.betaj[j,c]*(1-R[i]**2)/self.r[c]**2
+                        # eq 41 in paper has a small mistake:  (1-R^2)/r^2 should be (1-R^2/r^2) as below
+                        betaterm2 = 1-self.betaj[j,c]*(1-R[i]**2/self.r[c]**2)
 
                     v2int = simps(betaterm1*self.rhoj[j,c]*self.v2rj[j,c], x=z)
-                    v2zj[j,i] = abs(2.0*v2int)
-                    v2zj[j,i] /= Sigmaj[j,i]
+                    v2pj[j,i] = abs(2.0*v2int)
+                    v2pj[j,i] /= Sigmaj[j,i]
 
                     v2int = simps(betaterm2*self.rhoj[j,c]*self.v2rj[j,c], x=z)
                     v2Rj[j,i] = abs(2.0*v2int)
@@ -769,16 +926,112 @@ class limepy:
                     v2Tj[j,i] /= Sigmaj[j,i]
 
         self.R, self.Sigma = R, Sigma
-        self.v2z, self.v2R, self.v2T = v2z, v2R, v2T
+        self.v2p, self.v2R, self.v2T = v2p, v2R, v2T
         self.mcp = mcp
 
-        # Compute half-mass radii in projection
+        # TBD: Compute half-mass radii in projection
 
         if (self.multi):
             self.Sigmaj = Sigmaj
-            self.v2zj, self.v2RjR, self.v2Tj =  v2zj, v2Rj, v2Tj
+            self.v2pj, self.v2Rj, self.v2Tj =  v2pj, v2Rj, v2Tj
+
+
         return
 
+    def get_Paz(self, az_data, R_data, jns):
+        """
+        Computes probability of line of sight acceleration at projected R : P(az|R)
+        """
+
+        # Under construction !!!
+
+        # Return P(az|R)
+
+        az_data = abs(az_data) # Consider only positive values
+
+        # Assumes units of az [m/s^2] if self.G ==0.004302, else models units
+        # Conversion factor from [pc (km/s)^2/Msun] -> [m/s^2]
+        az_fac = 1./3.0857e10 if (self.G==0.004302) else 1
+
+        if (R_data < self.rt):
+            nz = self.nstep                   # Number of z value equal to number of r values
+            zt = sqrt(self.rt**2 - R_data**2) # maximum z value at R
+
+            z = numpy.logspace(log10(self.r[1]), log10(zt), nz)
+
+            spl_Mr = UnivariateSpline(self.r, self.mc, s=0, ext=1)  # Spline for enclosed mass
+
+            r = sqrt(R_data**2 + z**2)                        # Local r array
+            az = self.G*spl_Mr(r)*z/r**3                     # Acceleration along los
+            az[-1] = self.G*spl_Mr(self.rt)*zt/self.rt**3    # Ensure non-zero final data point
+
+            az *= az_fac # convert to [m/s^2]
+            az_spl = UnivariateSpline(z, az, k=4, s=0, ext=1) # 4th order needed to find max (can be done easier?)
+
+            zmax = az_spl.derivative().roots()  # z where az = max(az), can be done without 4th order spline?
+            azt = az[-1]                        # acceleration at the max(z) = sqrt(r_t**2 - R**2)
+
+            # Setup spline for rho(z)
+            if jns == 0 and self.nmbin == 1:
+                rho = self.rho
+            else:
+                rho = self.rhoj[jns]
+
+            rho_spl = UnivariateSpline(self.r, rho, ext=1, s=0)
+            rhoz = rho_spl(sqrt(z**2 + R_data**2))
+            rhoz_spl = UnivariateSpline(z, rhoz, ext=1, s=0)
+
+            # Now compute P(a_z|R)
+            # There are 2 possibilities depending on R:
+            #  (1) the maximum acceleration occurs within the cluster boundary, or
+            #  (2) max(a_z) = a_z,t (this happens when R ~ r_t)
+
+            nr, k = nz, 3 # bit of experimenting
+
+            # Option (1): zmax < max(z)
+            if len(zmax)>0:
+                zmax = zmax[0] # Take first entry for the rare cases with multiple peaks
+                # Set up 2 splines for the inverse z(a_z) for z < zmax and z > zmax
+                z1 = numpy.linspace(z[0], zmax, nr)
+                z2 = (numpy.linspace(zmax, z[-1], nr))[::-1] # Reverse z for ascending az
+
+                z1_spl = UnivariateSpline(az_spl(z1), z1, k=k, s=0, ext=1)
+                z2_spl = UnivariateSpline(az_spl(z2), z2, k=k, s=0, ext=1)
+
+            # Option 2: zmax = max(z)
+            else:
+                zmax = z[-1]
+                z1 = numpy.linspace(z[0], zmax, nr)
+                z1_spl = UnivariateSpline(az_spl(z1), z1, k=k, s=0, ext=1)
+
+            # Maximum acceleration along this los
+            azmax = az_spl(zmax)
+
+            # Now determine P(az_data|R)
+            if (az_data < azmax):
+                z1 = max([z1_spl(az_data), z[0]]) # first radius where az = az_data
+                Paz = rhoz_spl(z1)/abs(az_spl.derivatives(z1)[1])
+
+                if (az_data> azt):
+                    # Find z where a_z = a_z,t
+                    z2 = z2_spl(az_data)
+                    Paz += rhoz_spl(z2)/abs(az_spl.derivatives(z2)[1])
+
+                # Normalize to 1
+                Paz /= rhoz_spl.integral(0, zt)
+                self.z = z
+                self.az = az
+                self.Paz = Paz
+                self.azmax = azmax
+                self.zmax = zmax
+            else:
+                self.Paz = 0
+        else:
+            self.Paz = 0
+
+        return
+
+
     def interp_phi(self, r):
         """ Returns interpolated potential at r, works on scalar and arrays """
 
@@ -795,16 +1048,13 @@ class limepy:
         """
         Returns the value of the normalised DF at a given position in phase
         space, can only be called after solving Poisson's equation
-
         Arguments can be:
           - r, v                   (isotropic single-mass models)
           - r, v, j                (isotropic multi-mass models)
           - r, v, theta, j         (anisotropic models)
           - x, y, z, vx, vy, vz, j (all models)
-
         Here j specifies the mass bin, j=0 for single mass
         Works with scalar and array input
-
         """
 
         if (len(arg)<2) or (len(arg)==5) or (len(arg)==6) or (len(arg)>7):
@@ -842,14 +1092,16 @@ class limepy:
         DF = numpy.zeros([max(r.size, v.size)])
 
         if (len(r) == len(v2)):
-            c = (r<self.rt) and (v2<vesc2)
+            # Bitwise & needed to allow for possibility that r and v2 are arrays
+            c = (r<self.rt) & (v2<vesc2)
 
         if (len(r) == 1) and (len(v2) > 1):
             c = (v2<vesc2)
             if (r>self.rt): c=False
 
         if (len(r) > 1) and (len(v2) == 1):
-            c = (r<self.rt) and (numpy.zeros(len(r))+v2<vesc2)
+            # Bitwise & needed to allow for possibility that r and v2 are arrays
+            c = (r<self.rt) & (numpy.zeros(len(r))+v2<vesc2)
 
         if (sum(c)>0):
             # Compute the DF: equation (1), GZ15
@@ -868,9 +1120,6 @@ class limepy:
             DF[c] *= self.A[j]
 
         else:
-            DF = numpy.zeros(max(len(r),len(v)))
+	        DF = numpy.zeros(max(len(r),len(v)))
 
         return DF
-
-
-
